// ********************************
// *                              *
// * Styx Nova Intelligence Corp. *
// *           presents:          *
// *                              *
// *  .------------------------.  *
// *  | Cloudius Optimus Prime |  *
// *  `------------------------'  *
// *                              *
// * Copyright (c) 1996 Fredrik W *
// *  All world rights reserved.  *
// *                              *
// ********************************

#include <graphics.h>
#include <fstream.h>
#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <conio.h>
#include <dos.h>
#include <time.h>

#undef COLOURSHIFT

const char ver[] = "$VER: cloudius optimus prime 2.00 Copyright (c)1996 Fredrik Wangel";

//**************************************************************//

typedef unsigned char byte;
typedef unsigned  int word;
typedef unsigned long lint;

typedef struct {							//
	 byte   rgbBlue;						//
	 byte   rgbGreen;                //
	 byte   rgbRed;						//
	 byte   rgbReserved;					// always set to 0
} rgbQuad;

typedef struct {
	word bfType;							// 0x4d42
	lint bfSize;							// filesize in bytes (bfOffBits + (long(biWidth * biHeight * biBitCount) / 8))
	word bfReserved1;						// 0                                          ^
	word bfReserved2;						// 0                                          |
	lint bfOffBits;						// 14 + 2 ( <- pad ) + 40 + 4 * colours                      |
	lint biSize;							// 40                                         |
	long biWidth;							// pixels on x-axis                           |
	long biHeight;							// pixels on y-axis                           |
	word biPlanes;							// bitplanes ( =1 )                           |
	word biBitCount;						// bits per pixel ( =8 )                      |
	lint biCompression;					// 0 ( =BI_RGB )                              |
	lint biSizeImage;						// size of bitmap in bytes
	long biXPelsPerMeter;				// pixels per meter on x-axis (eg 3200)
	long biYPelsPerMeter;				// pixels per meter on x-axis (eg 2400)
	lint biClrUsed;						// 0 ( =all palette entries are in use)
	lint biClrImportant;					// 0 ( =all colours are important)
} bitmapInfo;


const char GrPath[]	= "";
const word BMP_TYPE	= 0x4d42;		// "MB" (written as BM in the file)
const lint BI_RGB		= 0;				// No compression

char		filename[128] = "";
int		maxcolor, maxx, maxy;
int		StartBlockSize;
rgbQuad	palette[256];


//**************************************************************//



#ifdef COLOURSHIFT

void waitdisplay(void)
{
  while((inportb(0x03DA) & 0x08) == 0);
  while((inportb(0x03DA) & 0x08) != 0);
}

void waitretrace(void)
{
  while((inportb(0x3DA) & 0x08) != 0);
  while((inportb(0x3DA) & 0x08) == 0);
}

void getpalrgb(byte c, byte &r, byte &g, byte &b)
{
	outportb(0x03c7, c);
	r = inportb(0x03c9);
	g = inportb(0x03c9);
	b = inportb(0x03c9);
}

void setpalrgb(byte c, byte r, byte g, byte b)
{
	outportb(0x03c8, c);
	outportb(0x03c9, r);
	outportb(0x03c9, g);
	outportb(0x03c9, b);
}

void shiftpalette()
{
	byte r, g, b;
	byte r0, g0, b0;

	getpalrgb(0, r0, g0, b0);
	for(register byte i = 1; i < maxcolor; i++)
	{
		getpalrgb(maxcolor - i + 1, r, g, b);
		getpalrgb(i, r0, g0, b0);
		setpalrgb(i, r, g, b);
		setpalrgb(i - 1, r0, g0, b0);
		if(kbhit())
			break;
	}
	setpalrgb(0, r0, g0, b0);
}

#endif /* COLOURSHIFT */



int bigEndian(int wrd)
{
	return ((wrd & 0x00ff) << 8) + ((wrd & 0xff00) >> 8);
}

int initscreen()
{
	int gdriver = 0, gmode = 2, errorcode;

	gdriver = installuserdriver("SVGA256", NULL);
	initgraph(&gdriver, &gmode, "");
	errorcode = graphresult();
	if (errorcode != grOk)
	{
		cout << grapherrormsg(errorcode) << " " << errorcode << endl;
		return 0;
	}
	maxcolor = getmaxcolor();
	maxx = getmaxx();
	maxy = getmaxy();
	return 1;
}

void usage()
{
	cout  << ver << endl << endl
         << "NAME\n  cop - generates fluffy cloud like pictures\n" << endl
			<< "SYNOPSIS\n  cop [ -n ] [ imagename ] \n" << endl
			<< "DESCRIPTION\n  cop will genrate pictures of clouds and display them" << endl
			<< "  on the screen, unless imagename is stated, in wich" << endl
			<< "  case a Windows Bitmap file (1) is created." << endl
			<< "  n is an integer between 1 and 160 and sets the starting" << endl
			<< "  blocksize. If n is small, the clouds will look strange." << endl
			<< "  n defaults to 80." << endl << endl
			<< "(1) Normal DIB 8bit with no compression" << endl;
}

void cleanup()
{
	closegraph();
}

void adjustpalette()
{
	for(int i = 0; i < maxcolor + 1; i++)
	{
		setrgbpalette(i, i / 4, 31 + i / 8, 63);
		palette[i].rgbRed			= i;
		palette[i].rgbGreen		= 128 + i / 2;
		palette[i].rgbBlue		= 255;
		palette[i].rgbReserved	= 0;
	}
}

int process()
{
	int blocksize = StartBlockSize;
	int colour;
	adjustpalette();
	do
	{
		for(register int y = 0; y < (maxy / blocksize) + 1; y++)
		{
			for(register int x = 0; x < (maxx / blocksize) + 1; x++)
			{
				if(kbhit())
				{
					nosound();
					getch();
					return 5;
				}
				if(blocksize == StartBlockSize)
				{
					colour = random(maxcolor + 1);
					if(random(5000) > 3500)
						colour = 1 + maxcolor - random(maxcolor / (1 + random(8)));
					colour %= (maxcolor + 1);
					setfillstyle(SOLID_FILL, colour);
				}
				else
				{
					colour = getpixel(x * blocksize >= maxx?
											0:
											x * blocksize + blocksize,
											y * blocksize);
					colour += getpixel(x * blocksize,
											y * blocksize < 2 * blocksize?
											maxy:
											y * blocksize - blocksize);
					colour += getpixel(x * blocksize < 2 * blocksize?
											maxx:
											x * blocksize - blocksize,
											y * blocksize);
					colour += getpixel(x * blocksize,
											y * blocksize > maxy - blocksize?
											0:
											y * blocksize + blocksize);
					colour /= 4;
					setfillstyle(SOLID_FILL, colour);
				}
				if(blocksize != 1)
					bar(x * blocksize, y * blocksize,
						 x * blocksize + blocksize - 1, y * blocksize + blocksize - 1);
				else
					putpixel(x, y, colour);
			}
		}
		blocksize /= 2;
	}
	while(blocksize > 0);
	nosound();

	sound(2500);
	delay(50);
	nosound();

#ifdef COLOURSHIFT
	while(!kbhit())
		shiftpalette();
#endif

// saving as Windows .BMP file (8-bit colour, no compression)

	if(filename[0])
	{
		bitmapInfo bmp;
		byte linedata[1024];
		ofstream fout;
		word padding = 0;

		fout.open(filename, ios::out | ios::binary);
		if(fout.fail())
			return 20;

		bmp.bfType				= BMP_TYPE;
		bmp.bfSize				= 14 + 2 + 40 + 4 * 256 + long(maxx + 1) * long(maxy + 1);
		bmp.bfReserved1		= 0;
		bmp.bfReserved2		= 0;
		bmp.bfOffBits			= 14 + 2 + 40 + 4 * 256;

		bmp.biSize				= 40;
		bmp.biWidth				= maxx + 1;
		bmp.biHeight			= maxy + 1;
		bmp.biPlanes			= 1;
		bmp.biBitCount			= 8;
		bmp.biCompression		= BI_RGB;
		bmp.biSizeImage		= long(maxx + 1) * long(maxy + 1);
		bmp.biXPelsPerMeter	= 3200;
		bmp.biYPelsPerMeter	= 2400;
		bmp.biClrUsed			= 0;
		bmp.biClrImportant	= 0;

		if(!fout.write((char *)&bmp, sizeof(bitmapInfo)))
		{
			fout.close();
			return 20;
		}

		if(!fout.write((char *)&palette, sizeof(rgbQuad) * (maxcolor + 1)))
		{
			fout.close();
			return 20;
		}

		if(!fout.write((char *)&padding, sizeof(padding)))
		{
			fout.close();
			return 20;
		}

		for(register int y = maxy; y >= 0; y--)
		{
			for(register int x = 0; x < maxx + 1; x++)
				linedata[x] = byte(getpixel(x, y));
			if(!fout.write((char *)&linedata, maxx + 1))
			{
				fout.close();
				return 20;
			}
			setcolor(0);
			line(0, y, maxx, y);
		}

		sound(2500);
		delay(25);
		nosound();

		fout.close();
	}
	else
		getch();

	return 0;
}

int main(int argc, char *argv[])
{
	StartBlockSize = 80;
	argc--;
	if(argc > 0)
	{
		if((!strcmp("-h", argv[1])) ||
			(!strcmp("-H", argv[1])) ||
			(!strcmp("-?", argv[1])) ||
			(!strcmp("/?", argv[1])))
		{
			usage();
			return 5;
		}
		else if(*argv[1] == '-' || *argv[1] == '/')
		{
			if(isdigit((char)argv[1][1]))
				StartBlockSize = atoi(argv[1] + 1);
			if(argc > 1)
				strcpy(filename, argv[2]);
		}
		else if(*argv[2] == '-' || *argv[2] == '/')
		{
			if(isdigit((char)argv[2][1]))
				StartBlockSize = atoi(argv[2] + 1);
			strcpy(filename, argv[1]);
		}
		else
			strcpy(filename, argv[1]);
	}
	if(StartBlockSize > 160 || StartBlockSize < 1)
		StartBlockSize = 80;
	randomize();
	if(!initscreen())
	{
		cout << "Error initializing graphics" << endl;
		return 20;
	}
	if(process())
	{
		cleanup();
		cout << "User interrupted!" << endl;
		return 5;
	}
	cleanup();
	return 0;
}
