/**************************************************************
   This example shows how to set up a basic device and viewport
   ,set up primitives, clut, texture page and render multiple
	affine lit texture mapped primitives utilizing macros.

	Note.
	This is rather a slow method and is not really recommended
	in serious applications, this is meant as a tutorial program
	demonstrating the various aspects of zeus.
**************************************************************/

#include "..\..\..\h\z_all.h"

#define		NAME			"Mr DD Demo 1"

#define		SCREEN_RES_X	320
#define		SCREEN_RES_Y	200

#define		MAX_CUBES		32


int			app_active;
int			installed=-1;

/* object variables */
	VECTOR			object_trans;
	SVECTOR			object_angle;
	MATRIX			object_matrix;


/* Cube data */
	/* vertices positions relative to cubes origiin */
	SVECTOR			cube_verts[8]={{-128,-128,-128},
									{128,-128,-128},
									{-128,-128,128},
									{128,-128,128},
									{-128,128,-128},
									{128,128,-128},
									{-128,128,128},
									{128,128,128}
									};

	/* Face indices into vertex list (cube_verts) 4 per face */
	short			cube_indices[6*4]={	0,1,3,2,	/* TOP */
										5,4,6,7,	/* BOTTOM */
										2,3,7,6,	/* FRONT */
										1,0,4,5,	/* BACK */
										0,2,6,4,	/* LEFT */
										3,1,5,7		/* RIGHT */
										};

	SVECTOR			cube_normals[6] = {	{0, -ONE, 0},
										{0, ONE, 0},
										{0, 0, -ONE},
										{0, 0, ONE},
										{-ONE, 0, 0},
										{ONE, 0, 0}};

	VECTOR			random_trans[MAX_CUBES];
	SVECTOR			random_angles[MAX_CUBES];
	SVECTOR			random_angle_vels[MAX_CUBES];

/* Light source */
	MATRIX			cmat={
							MAX_SHADE, MAX_SHADE, MAX_SHADE,
							0,0,0,
							0,0,0,
							0,0,0	/* translation (none)*/
							};

	MATRIX			light;
	SVECTOR			lgtang={0,2048,0};

/* Device and viewport */
	DEVICE_TYPE		device;
	VIEWPORT_TYPE	view;

	char			*ramp_clut;
	VERTEX			*next_vertex_ptr;
	POLYGON			*next_face_ptr;
	u_short			*next_index_ptr;
	u_short			next_vertex_no;

void create_random_cubes(VECTOR *trans, SVECTOR *angles, SVECTOR *angle_vels, int num_cubes)
{
	int		t;

	for (t=0; t<num_cubes; t++)
	{	
		/* Create a random translation in space */
		trans->vx=((rand()-16384) >> 4);
		trans->vy=((rand()-16384) >> 4);
		trans->vz=((rand()-16384) >> 4) + 1500;

		angles->vx=rand()&4095;
		angles->vy=rand()&4095;
		angles->vz=rand()&4095;

		angle_vels->vx=((rand()-16384) >> 8);
		angle_vels->vy=((rand()-16384) >> 8);
		angle_vels->vz=((rand()-16384) >> 8);

		angles++;
		angle_vels++;
		trans++;
	}
}

void draw_cube(SVECTOR *v, short *i, SVECTOR *normal_list, int nf)
{
	int			t;
	SVECTOR		*v1, *v2, *v3, *v4;
	short		index;
	int			cull;
	long		ot_pos;
	long		p, flag;
	SVECTOR		r_normal;	/* Rotated normal */
	CVECTOR		colors={MAX_SHADE/2, MAX_SHADE/2, MAX_SHADE/2};	/* Base shade */

	/* Do all faces */
	for (t=0; t<nf; t++)
	{
		/* point to source face vertices */
		index=*i;
		v1=v + index;
		index=*(i+1);
		v2=v + index;
		index=*(i+2);
		v3=v + index;
		index=*(i+3);
		v4=v + index;

		/* Apply transformation to vertices (SONY PLAYSTATION style) */
		cull=RotAverageNclip4(v1, v2, v3, v4, (long *)next_vertex_ptr,
			(long *)(next_vertex_ptr+1), (long *)(next_vertex_ptr+2),
			(long *)(next_vertex_ptr+3), &p, &ot_pos, &flag);

		/* Only proceed if face is not facing away from us */
		if (cull<0)
		{
			/* tag is always set to NULL for order table purposes */
			next_face_ptr->tag=NULL;

			/* Rotate normal */
			RotSV(normal_list, &r_normal, &flag);

			/* Calculate shade of face */
			NormalColorCol(&r_normal, &colors, (CVECTOR *)&(next_face_ptr->r));

			/* setup number of vertices */
			next_face_ptr->nv=QUAD;

			/* setup primitive vertices */
			next_face_ptr->v=next_vertex_ptr;

			/* setup face index table */
			next_face_ptr->face_data=next_index_ptr;

			/* setup indices to primitive vertices */
			set_index4(next_index_ptr);

			/* setup polygon type */
			next_face_ptr->id=MTEXTURED|MRAMP;

			next_face_ptr->rgb=0;
			next_face_ptr->tpage=0;
			next_face_ptr->semi=0;

			/* set up texture coaardinates */
			set_uvwh4(next_vertex_ptr, 0, 0, FP_TEXT(64)-1, FP_TEXT(64)-1);
			next_vertex_ptr+=QUAD;

			/* scale ot_pos down a little to keep within range of order table */
			ot_pos>>=2;

			/* Add primitive to viewport */
			/* Zeus does support AddPrim() but you must get the order table
			   address from the viewport ie view.ot and check for a primitive
			   going out of range (view.ot_size - size of order table) */
			VIEWPORT_add_prim(&view, next_face_ptr++, ot_pos);
		}

		/* Move to next set of source indices */
		i+=QUAD;

		/* Move to next normal */
		normal_list++;
	}
}


/*************************************************/
/* Main loop responsible for rotating models etc */
/*************************************************/
BOOL main_loop()
{
	char	err_str[16];
	int		err, t;
	VECTOR	*object_trans_list=&random_trans[0];
	SVECTOR	*object_angles_list=&random_angles[0];
	SVECTOR	*object_angle_vels=&random_angle_vels[0];
	VECTOR	object_trans_r;
	SVECTOR	object_angle_r;

	/* Rotate light source */
/*	lgtang.vy+=16;*/
/*	lgtang.vy&=4095;*/

	/* Create constant light matrix */
	RotMatrixYXZ(&lgtang, &light);
	SetLightMatrix(&light);

	/* Spin dry and draw 32 cubes at random positions */
	/* Not a very efficient way of doing it */
	for (t=0; t<MAX_CUBES; t++)
	{
		/* Spin object */
		object_angles_list->vx+=object_angle_vels->vx;
		object_angles_list->vy+=object_angle_vels->vy;
		object_angles_list->vz+=object_angle_vels->vz;
		object_angles_list->vx&=4095;
		object_angles_list->vy&=4095;
		object_angles_list->vz&=4095;

		/* Set translation from random generated translations */
		object_trans_r.vx=object_trans.vx + object_trans_list->vx;
		object_trans_r.vy=object_trans.vy + object_trans_list->vy; 
		object_trans_r.vz=object_trans.vz + object_trans_list->vz;

		/* Set orientation from random generated orientations */
		object_angle_r.vx=(object_angle.vx + object_angles_list->vx)&4095;
		object_angle_r.vy=(object_angle.vy + object_angles_list->vy)&4095;
		object_angle_r.vz=(object_angle.vz + object_angles_list->vz)&4095;

		/* Create rotation and translation matrix for object */
		RotMatrix(&object_angle_r, &object_matrix);
		TransMatrix(&object_matrix, &object_trans_r);

		/* Set as constant rotation and translation matrix */
		SetTransMatrix(&object_matrix);
		SetRotMatrix(&object_matrix);

		/* Draw the object */
		draw_cube(&cube_verts[0], &cube_indices[0], &cube_normals[0], 6);

		/* Move to next set of random data */
		object_trans_list++;
		object_angles_list++;
		object_angle_vels++;
	}

	/* Process the device and all viewports etc.. connected to it */
	err=DEVICE_process();
	if (err!=DEVICE_OK)
	{
		printf("DEVICE ERROR %d", err);

		return(FALSE);
	}
	frame_count++;

	return(TRUE);
}
void cleanup_all()
{
	/* Close down device, all viewports, order tables, primitive data, palettes etc.. */
	DEVICE_delete(&device);
}

BOOL setup_all()
{
	int					err;
	char				err_str[16];
	_RECT				view_dim;
	int					t;

	/* This is just so I can keep track of number of game frames */
	frame_count=0;

	/* Initialize zeus */
	if (SYS_init_zeus()==NULL)
	{
		printf("Could not initialize zeus\n");
		return(FALSE);
	}

	/* Request memory texture page 256 lines high */
	tpage0=SYS_request_texture_page_mem(1*256);
	if (tpage0==NULL)
	{
		printf("Not enough memory for texture page\n");
		exit(1);
	}
	/* Request a clut (FLAT color model) */
	ramp_clut=SYS_request_clut_mem();
	if (ramp_clut==NULL)
	{
		printf("Not enough memory for ramp clut\n");
		return(FALSE);
	}
	
	/* Request 20,000 graphics primitives */
	if (SYS_request_n_quads((int *)&next_index_ptr, (int *)&next_face_ptr, (int *)&next_vertex_ptr, 20000)==NULL)
	{
		printf("Not enough memory for quadrangles\n");
		return(FALSE);
	}

	/* Clear device structure */
	DEVICE_clear(&device);

	/* Create primary device */
	err=DEVICE_create(&device, SCREEN_RES_X, SCREEN_RES_Y, 8, NULL,
			0, VIDEO_RAM_BASE_PC, DEVICE_AUTO_CLEAR, NULL, 1);
	if (err!=DEVICE_OK)
	{
		printf("DEVICE ERROR %d\n"), err;

		return(FALSE);
	}

	/* Turn vertical sync off */
	DEVICE_vertical_sync_off(&device);

	/* Load palette and set */
	DEVICE_load_palette(&device, "../newpal.pal");
	DEVICE_set_palette(&device);

	/* Create us a viewport */
	VIEWPORT_clear(&view);
	view_dim.x=0;
	view_dim.y=0;
	view_dim.w=SCREEN_RES_X-1;
	view_dim.h=SCREEN_RES_Y-1;
	err=VIEWPORT_create(&view, &view_dim, 0, FP7, 1024, DEVICE_NO_AUTO_CLEAR);
	if (err!=VIEWPORT_OK)
	{
		printf("VIEWPORT ERROR %d\n", err);

		return(FALSE);
	}
	VIEWPORT_set_border(&view, 15);			/* Tells viewport to display a border */
	VIEWPORT_set_min_size(&view, 32,8);		/* Minimized size */
	VIEWPORT_set_max_size(&view, 320,200);	/* Maximized size */
	VIEWPORT_set_near_z(&view, 0);			/* Font Z clipping plane value */
	VIEWPORT_set_interpolation(&view, 4);	/* interpolation during perspective correct texture mapped rendering */
	VIEWPORT_set_perspective(&view, 8);		/* Set eye distance from screen */


	/* Assign texture page to viewport */
	VIEWPORT_assign_tpage(&view, tpage0);

	/* Assign clut to viewport */
	VIEWPORT_assign_ramp_clut(&view, ramp_clut);

	/* Assign primitives to viewport */
	VIEWPORT_assign_prims(&view, (int *)&next_index_ptr,
		(int *)&next_face_ptr, (int *)&next_vertex_ptr, (int *)&next_vertex_no);

	/* Add the viewport to device */
	DEVICE_add_viewport(&device, &view);
				 
	/* Make 'device' the current working device */
	if (err=DEVICE_make_current(&device)<0)
	{
		printf("COULD NOT install devive \n");

		return(FALSE);
	}

	/* Make 'view' current working viewport */
	VIEWPORT_make_current(&view);

	/* Always call at least once before doing anything with it */
	DEVICE_process();

	/* Load ramp clut */
	FILE_load_clut("../pal_ref1.tab", ramp_clut);

	/* Load texture raw date */
	FILE_load_tpage("../nw12.bin", tpage0);

	/* Set objects position in space */
	object_trans.vx=0;
	object_trans.vy=0;
	object_trans.vz=512;

	/* Create random cube translation list */
	create_random_cubes(random_trans, random_angles, random_angle_vels, MAX_CUBES);

	/* Set constant color matrix */
	SetColorMatrix(&cmat);

	/* Rotate ambient light */
	SetBackColor(MAX_SHADE/2, MAX_SHADE/2, MAX_SHADE/2);

	/* Set color to depth queue */
	SetFarColor(0,0,0);

	return(TRUE);
}

void main()
{
	/* Make a device, viewport and load in graphics files etc */
	if (!(setup_all()))
	    return;

	installed=TRUE;

	/* Install keyboard handler */
	KBD_install();

	/* MAIN LOOP CALL */
	do
	{ 
		if (!main_loop())
			return;
	}
	while(!keybuf[KEY_ESC]);	/* continue until escape key pressed */

	/* kill keyboard handler */
	KBD_kill();

	cleanup_all();

	set_video_mode(3);			/* set text video mode */
}
