#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <mem.h>
#include <math.h>
#include "hscreen.h"
#include "hdib.h"

HIDDEN_SCREEN::HIDDEN_SCREEN(HDC hdc, int width, int height)
{
	BMPINFO Info;

	Width = (WORD)width;
	Height = (WORD)height;

	Orientation = -1;//(int)Info.bmiHeader.biHeight;
// Use Top-Down for easier access to bytes

	Info.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
	Info.bmiHeader.biWidth = Width;
	Info.bmiHeader.biHeight = -Height;
	Info.bmiHeader.biPlanes = 1;
	Info.bmiHeader.biBitCount = 8;
	Info.bmiHeader.biCompression = BI_RGB;
	Info.bmiHeader.biSizeImage = Width * Height;
	Info.bmiHeader.biXPelsPerMeter = 0;
	Info.bmiHeader.biYPelsPerMeter = 0;
	Info.bmiHeader.biClrUsed = 256;
	Info.bmiHeader.biClrImportant = 256;

	HiddenBMP = CreateDIBSection(hdc, (BITMAPINFO *)&Info, DIB_PAL_COLORS,
						  &Surface, NULL, 0);
	MainPal = CreateHalftonePalette(hdc);
	HiddenDC = CreateCompatibleDC(hdc);
	OldBMP = SelectObject(HiddenDC, HiddenBMP);

	GetPaletteEntries(MainPal, 0, 256, (PALETTEENTRY far *)Info.bmiColors);
	for(int Counter = 0; Counter < 256; Counter++)
	{
		BYTE Temp = Info.bmiColors[Counter].rgbBlue;
		Info.bmiColors[Counter].rgbBlue = Info.bmiColors[Counter].rgbRed;
		Info.bmiColors[Counter].rgbRed = Temp;
	}

	PatBlt(HiddenDC, 0, 0, width, height, WHITENESS);

	BmpInfo = Info;
};

HIDDEN_SCREEN::~HIDDEN_SCREEN()
{
	SelectObject(HiddenDC, OldBMP);
	DeleteObject(HiddenBMP);
	DeleteDC(HiddenDC);
	DeleteObject(MainPal);
};

int HIDDEN_SCREEN::BltToScreen(HDC Screen, int x, int y)
{
	BitBlt(Screen, x, y, Width, Height, HiddenDC, 0, 0, SRCCOPY);
	return 0;
};

int HIDDEN_SCREEN::StretchToScreen(HDC Screen, int x, int y, int w, int h)
{
	StretchBlt(Screen, x, y, w, h, HiddenDC, 0, 0, Width, Height, SRCCOPY);
	return 0;
};

int HIDDEN_SCREEN::Clear()
{
	PatBlt(HiddenDC, 0, 0, Width, Height, BLACKNESS);
	return 0;
};

int HIDDEN_SCREEN::Clear(COLORREF Col)
{
	HBRUSH Brush, Oldbrush;

	Brush = CreateSolidBrush(Col);
	Oldbrush = SelectObject(HiddenDC, Brush);
	PatBlt(HiddenDC, 0, 0, Width, Height, PATCOPY);
	SelectObject(HiddenDC, Oldbrush);
	DeleteObject(Brush);
	return 0;
};

int HIDDEN_SCREEN::UseDCPal(HDC hdc)
{
	RGBQUAD Color[256];
	int i;
	BYTE Temp;
	
	GetSystemPaletteEntries(hdc, 0, 256, (PALETTEENTRY *)Color);
	for(i = 0; i < 256; i++)
	{
		Temp = Color[i].rgbBlue;
		Color[i].rgbBlue = Color[i].rgbRed;
		Color[i].rgbRed = Temp;
	}
	SetDIBColorTable(HiddenDC, 0, 256, Color);
	return 0;
};
		
	
int HIDDEN_SCREEN::UseHalftonePalette(HDC hdc)
{
	HPALETTE Pal, Old;
	RGBQUAD	Colors[256];

	Pal = CreateHalftonePalette(hdc);
	Old = SelectPalette(hdc, Pal, FALSE);
	RealizePalette(hdc);
	DeleteObject(Old);

	GetPaletteEntries(Pal, 0, 256, (PALETTEENTRY far *)Colors);
	for(int i = 0; i < 256; i++)
	{
		BYTE Temp = Colors[i].rgbBlue;
		Colors[i].rgbBlue = Colors[i].rgbRed;
		Colors[i].rgbRed = Temp;
	}
	SetDIBColorTable(HiddenDC, 0, 256, Colors);
	return 0;
};

int HIDDEN_SCREEN::DisplayBitmap(HDIB Bitmap, int xPos, int yPos)
{
	char 			*c, *d;
	int 			WidthCnt, HeightCnt;
	int 			WidthIndex, HeightIndex;
	int				i;

	WidthCnt = (int)Bitmap.Info.biWidth;
	WidthIndex = 0;
	HeightCnt = (int)Bitmap.Info.biHeight;
	HeightIndex = 0;

	if((xPos > Width) || ((xPos + Bitmap.Info.biWidth) < 0)
	|| (yPos > Height) || ((yPos + Bitmap.Info.biHeight) < 0))
		return 3;
	if(xPos < 0)
	{
		WidthCnt = (int)Bitmap.Info.biWidth + xPos;
		WidthIndex = -xPos;
	}
	if(xPos + (int)Bitmap.Info.biWidth >= Width)
		WidthCnt = (int)Bitmap.Info.biWidth - (xPos + (int)Bitmap.Info.biWidth - Width);

	if(yPos < 0)
	{
		HeightCnt = (int)Bitmap.Info.biHeight + yPos;
		HeightIndex = -yPos;
	}
	if(yPos + (int)Bitmap.Info.biHeight >= Height)
		HeightCnt = (int)Bitmap.Info.biHeight - (yPos + (int)Bitmap.Info.biHeight - Height);

	c = (char *)Surface + ( (yPos + HeightIndex) * Width) + (xPos + WidthIndex);
	d = (LPSTR)Bitmap.Data + (HeightIndex) * (int)Bitmap.Info.biWidth + WidthIndex;
	for(i = 0; i < HeightCnt; i++)
	{
		memcpy(c, d, sizeof(char) * WidthCnt);
		c += Width;
		d += Bitmap.Info.biWidth;
	}
	return 1;
};

#pragma argsused
int HIDDEN_SCREEN::TDisplayBitmap(HDIB Bitmap, int xPos, int yPos, BYTE TColor)
{
	char 			*c, *d;
	int 			WidthCnt, HeightCnt;
	int 			WidthIndex, HeightIndex;
	int				i, j;

	WidthCnt = (int)Bitmap.Info.biWidth;
	WidthIndex = 0;
	HeightCnt = (int)Bitmap.Info.biHeight;
	HeightIndex = 0;

	if((xPos > Width) || ((xPos + Bitmap.Info.biWidth) < 0)
	|| (yPos > Height) || ((yPos + Bitmap.Info.biHeight) < 0))
		return 3;
	if(xPos < 0)
	{
		WidthCnt = (int)Bitmap.Info.biWidth + xPos;
		WidthIndex = -xPos;
	}
	if(xPos + (int)Bitmap.Info.biWidth >= Width)
		WidthCnt = (int)Bitmap.Info.biWidth - (xPos + (int)Bitmap.Info.biWidth - Width);

	if(yPos < 0)
	{
		HeightCnt = (int)Bitmap.Info.biHeight + yPos;
		HeightIndex = -yPos;
	}
	if(yPos + (int)Bitmap.Info.biHeight >= Height)
		HeightCnt = (int)Bitmap.Info.biHeight - (yPos + (int)Bitmap.Info.biHeight - Height);

	c = (char *)Surface + ( (yPos + HeightIndex) * Width) + (xPos + WidthIndex);
	d = (LPSTR)Bitmap.Data + (HeightIndex) * (int)Bitmap.Info.biWidth + WidthIndex;
	for(i = 0; i < HeightCnt; i++)
	{
		for(j = 0; j < (WidthCnt >> 3); j++)
		{
			if(*d != TColor)
				*c = *d;
			c++; d++;
			if(*d != TColor)
				*c = *d;
			c++; d++;
			if(*d != TColor)
				*c = *d;
			c++; d++;
			if(*d != TColor)
				*c = *d;
			c++; d++;
			if(*d != TColor)
				*c = *d;
			c++; d++;
			if(*d != TColor)
				*c = *d;
			c++; d++;
			if(*d != TColor)
				*c = *d;
			c++; d++;
			if(*d != TColor)
				*c = *d;
			c++; d++;
		}
		int someleft = WidthCnt & 0x7;
		while(someleft > 0)
		{
			if(*d != TColor)
				*c = *d;
			c++; d++;
			someleft--;
		}
		c += Width - WidthCnt;
		d += Bitmap.Info.biWidth - WidthCnt;
	}
	return TRUE;
};

// A useful function
void ClearSystemPalette(void)
{
	HDC 		 TempDC;
	HPALETTE     Pal, SavePal;
	NPLOGPALETTE Palette;

	TempDC = GetDC(0);
	Palette = (NPLOGPALETTE) LocalAlloc(LMEM_FIXED,
		(sizeof(LOGPALETTE) + (sizeof(PALETTEENTRY) * (256))));
	Palette->palVersion = 0x300;
	Palette->palNumEntries = 256;

	for (WORD Index = 0; Index < (WORD)256; Index++)
	{
		Palette->palPalEntry[Index].peRed =0;
		Palette->palPalEntry[Index].peGreen = 0;
		Palette->palPalEntry[Index].peBlue = 0;
		Palette->palPalEntry[Index].peFlags = (BYTE)PC_NOCOLLAPSE;
	}

	Pal = CreatePalette((LPLOGPALETTE)Palette);
	LocalFree((NPLOGPALETTE)Palette);
	SavePal = SelectPalette(TempDC, Pal, FALSE);
	RealizePalette(TempDC);
	Pal = SelectPalette(TempDC, SavePal, FALSE);
	ReleaseDC(0, TempDC);
	DeleteObject(Pal);
	DeleteObject(SavePal);
};

void AdjustClientRegion(HWND hWindow, int x1, int y1, int w, int h)
{
	RECT r;
	long NewWidth, NewHeight;

	GetClientRect(hWindow, &r);
	NewWidth = w - (r.right - r.left);
	NewHeight = h - (r.bottom - r.top);
	GetWindowRect(hWindow, &r);
	SetWindowPos(hWindow, HWND_TOP, 0, 0, r.right - r.left + NewWidth, r.bottom - r.top + NewHeight, SWP_NOZORDER);
};

typedef struct
{
	int StartX, EndX;
	POINT TStart, TEnd;
} TSCAN;

int HIDDEN_SCREEN::RotateAndTDisplayBitmap(HDIB Dib, int StartX, int StartY, int RotateX, int RotateY, float Rads, BYTE TColor)
{
	POINT	Corner[4];
	POINT	TextPt[4];
	int		i, j;
	float	sR, cR;
	TSCAN Scan[200];
	long tmp, y;
	long x, m;
	long ts1, ts2, tslope1, tslope2;
	char *c;
	long x1, y1, x2, y2;
	long val;
	POINT t1, t2, tp;
	int OldVal;
	
	sR = sin(Rads);
	cR = cos(Rads);

	Corner[0].x = StartX; Corner[0].y = StartY;
	Corner[1].x = StartX + Dib.Info.biWidth; Corner[1].y = StartY;
	Corner[2].x = StartX + Dib.Info.biWidth; Corner[2].y = StartY + Dib.Info.biHeight;
	Corner[3].x = StartX; Corner[3].y = StartY + Dib.Info.biHeight;

	for(i = 0; i < 4; i++)
	{
		x = Corner[i].x;
		y = Corner[i].y;
		Corner[i].x = x;
		Corner[i].y = y;
	}

	TextPt[0].x = 0;  TextPt[0].y = 0;
	TextPt[1].x = Dib.Info.biWidth - 1;  TextPt[1].y = 0;
	TextPt[2].x = Dib.Info.biWidth - 1;  TextPt[2].y = Dib.Info.biHeight - 1;
	TextPt[3].x = 0;  TextPt[3].y = Dib.Info.biHeight - 1;

	for(i = 0; i < 200; i++)
	{
		Scan[i].StartX = -32000;
		Scan[i].EndX = -32000;
	}

	j = 3;
	for(i = 0; i < 4; i++)
	{
		x1 = (long)Corner[j].x;
		y1 = (long)Corner[j].y;
		x2 = (long)Corner[i].x;
		y2 = (long)Corner[i].y;
		t1.x = TextPt[j].x;
		t2.x = TextPt[i].x;
		t1.y = TextPt[j].y;
		t2.y = TextPt[i].y;

		if(y2 != y1)
		{
			if(y2 < y1)
			{
				tmp = y2;
				y2 = y1;
				y1 = tmp;

				tmp = x2;
				x2 = x1;
				x1 = tmp;

				tp = t2;
				t2 = t1;
				t1 = tp;
			}

			x = (long)(x1 << 12);
			m = ((long)((x2 - x1) << 12)) / ((long)(y2 - y1));
			ts1 = (long)(t1.y << 14);
			ts2 = (long)(t1.x << 14);
			tslope1 = ((t2.y - t1.y) << 14) / ((long)(y2 - y1));
			tslope2 = ((t2.x - t1.x) << 14) / ((long)(y2 - y1));

			x += m;
			ts1 += tslope1;
			ts2 += tslope2;
			y1++;

			for(y = y1; y <= y2; y++)
			{
				if((y >= 0) && (y < 200))
				{
					if(Scan[y].StartX == -32000)
					{
						Scan[y].StartX = x >> 12;
						Scan[y].TStart.x = ts2 >> 14;
						Scan[y].TStart.y = ts1 >> 14;
					}
					else
					{
						Scan[y].EndX = x >> 12;
						Scan[y].TEnd.x = ts2 >> 14;
						Scan[y].TEnd.y = ts1 >> 14;
					}
				}
				x += m;
				ts1 += tslope1;
				ts2 += tslope2;
			}
		}

		j = i;
	}

	for(i = 0; i < 200; i++)
	{
		if(Scan[i].StartX != -32000)
		{
			if(Scan[i].EndX == -32000)
			{
				Scan[i].EndX = Scan[i].StartX;
				Scan[i].TEnd = Scan[i].TStart;
			}

			if(Scan[i].EndX < Scan[i].StartX)
			{
				tmp = Scan[i].EndX;
				Scan[i].EndX = Scan[i].StartX;
				Scan[i].StartX = tmp;

				tp = Scan[i].TEnd;
				Scan[i].TEnd = Scan[i].TStart;
				Scan[i].TStart = tp;
			}

			if((Scan[i].StartX >= 320) || (Scan[i].EndX < 0))
				continue;

			ts1 = (Scan[i].TStart.x << 14);
			ts2 = (Scan[i].TStart.y << 14);
			if((Scan[i].EndX - Scan[i].StartX) == 0)
			{
				tslope1 = 0;
				tslope2 = 0;
			}
			else
			{
				tslope1 = ((Scan[i].TEnd.x - Scan[i].TStart.x) << 14) / (Scan[i].EndX - Scan[i].StartX);
				tslope2 = ((Scan[i].TEnd.y - Scan[i].TStart.y) << 14) / (Scan[i].EndX - Scan[i].StartX);
			}

			if(Scan[i].StartX < 0)
			{
				ts1 += ((tslope1 * -Scan[i].StartX));
				ts2 += ((tslope2 * -Scan[i].StartX));
				Scan[i].StartX = 0;
			}

			if(Scan[i].EndX >= 320)
				Scan[i].EndX = 319;

			long indx = i * 320 + Scan[i].StartX;
			c = (char *)Surface + indx;
			char *tx = (LPSTR)Dib.Data;
			long Width = Dib.Info.biWidth;

			for(j = 0; j < ((Scan[i].EndX - Scan[i].StartX) >> 2); j++)
			{
				if((*(tx + ((ts2 >> 14) * Width) + (ts1 >> 14))) != 0)
					*c  = *(tx + ((ts2 >> 14) * Width) + (ts1 >> 14));
				c++;
				ts1 += tslope1;
				ts2 += tslope2;

				if((*(tx + ((ts2 >> 14) * Width) + (ts1 >> 14))) != 0)
					*c  = *(tx + ((ts2 >> 14) * Width) + (ts1 >> 14));
				c++;
				ts1 += tslope1;
				ts2 += tslope2;

				if((*(tx + ((ts2 >> 14) * Width) + (ts1 >> 14))) != 0)
					*c  = *(tx + ((ts2 >> 14) * Width) + (ts1 >> 14));
				c++;
				ts1 += tslope1;
				ts2 += tslope2;

				if((*(tx + ((ts2 >> 14) * Width) + (ts1 >> 14))) != 0)
					*c  = *(tx + ((ts2 >> 14) * Width) + (ts1 >> 14));
				c++;
				ts1 += tslope1;
				ts2 += tslope2;

			}

			if((Scan[i].EndX - Scan[i].StartX) & 0x3)
			{
				val = (Scan[i].EndX - Scan[i].StartX) & 0x3;
				while(val--)
				{
					if((*(tx + ((ts2 >> 14) * Width) + (ts1 >> 14))) != 0)
						*c  = *(tx + ((ts2 >> 14) * Width) + (ts1 >> 14));
					c++;
					ts1 += tslope1;
					ts2 += tslope2;
				}
			}

		}
	}
    return 0;
};
